package com.jiuqi.grid;

import java.io.Serializable;

public final class CurrencyCellProperty extends NumberCellProperty implements CurrencyCellPropertyIntf, CellDataProcessorIntf, Serializable
{
    public static final String[] CURRENCY_UNIT_NAMES;
    public static final String[] CURRENCY_UNIT_TITLES;
    public static final String[] CURRENCY_UNIT_SIGNS;
    
    static {
        CURRENCY_UNIT_NAMES = new String[] { "", "CNY", "USD", "EUR", "JPY", "HKD", "GBP", "CHF", "SUR", "CAD" };
        CURRENCY_UNIT_TITLES = new String[] { "", "人民币元", "美元", "欧元", "日圆", "港元", "英镑", "瑞士法郎", "俄罗斯卢布", "加拿大元" };
        CURRENCY_UNIT_SIGNS = new String[] { "", "￥", "＄", "EUR", "J.￥", "HK＄", "￡", "SF.", "Rbs.", "Can.＄" };
    }
    
    public CurrencyCellProperty(final GridCell cell) {
        this(new CellDataProperty(cell));
    }
    
    public CurrencyCellProperty(final CellDataPropertyIntf dataProperty) {
        super(dataProperty);
    }
    
    public int getUnitIndex() {
        final int dataFormat = this.cellDataProperty.getDataFormat();
        return dataFormat & 0xF;
    }
    
    public void setUnitIndex(final int value) {
        if (value < 0 || value >= 10) {
            throw new IllegalArgumentException("货币符号被错误设置,只能在0-9之间!");
        }
        this.cellDataProperty.setDataFormat((byte)value);
    }
    
    public int getUnitShowType() {
        return (this.cellDataProperty.getDataProperty() & 0x300) >> 8;
    }
    
    public void setUnitShowType(int value) {
        if (value < 0 || value > 3) {
            throw new IllegalArgumentException("显示类型被错误设置,应该被设置到0-3之间!");
        }
        value &= 0x3;
        this.cellDataProperty.setDataProPerty((short)((this.cellDataProperty.getDataProperty() & 0xFFFFFCFF) | value << 8));
    }
    
    public String getUnitTitle() {
        return CurrencyCellProperty.CURRENCY_UNIT_TITLES[this.getUnitIndex()];
    }
    
    public void setUnitTitle(final String value) {
        int i = 0;
        boolean found = false;
        while (!found) {
            if (value.equals(CurrencyCellProperty.CURRENCY_UNIT_TITLES[i])) {
                found = true;
                break;
            }
            ++i;
        }
        if (found) {
            this.setUnitIndex(i);
            return;
        }
        throw new IllegalArgumentException("没有这样的货币单位!");
    }
    
    public boolean getShowCurrencyBox() {
        return this.cellDataProperty.getDataType() == 3 && (this.cellDataProperty.getDataProperty() & 0x400) != 0x0;
    }
    
    public void setShowCurrencyBox(final boolean value) {
        if (this.cellDataProperty.getDataType() == 3) {
            if (value) {
                this.cellDataProperty.setDataProPerty((short)(this.cellDataProperty.getDataProperty() | 0x400));
            }
            else {
                this.cellDataProperty.setDataProPerty((short)(this.cellDataProperty.getDataProperty() & 0xFFFFFBFF));
            }
        }
    }
    
    public String getUnitString() {
        return CurrencyCellProperty.CURRENCY_UNIT_SIGNS[this.getUnitIndex()];
    }
    
    @Override
    public String getShowText(final String storeText) {
        String showText = super.getShowText(storeText);
        final int unitIndex = this.getUnitIndex();
        if (unitIndex >= 0) {
            final int unitShowType = this.getUnitShowType();
            switch (unitShowType) {
                case 0: {
                    showText = String.valueOf(CurrencyCellProperty.CURRENCY_UNIT_SIGNS[unitIndex]) + " " + showText;
                    break;
                }
                case 1: {
                    showText = String.valueOf(CurrencyCellProperty.CURRENCY_UNIT_NAMES[unitIndex]) + " " + showText;
                    break;
                }
                case 2: {
                    showText = String.valueOf(showText) + " " + CurrencyCellProperty.CURRENCY_UNIT_NAMES[unitIndex];
                    break;
                }
                case 3: {
                    showText = String.valueOf(CurrencyCellProperty.CURRENCY_UNIT_TITLES[unitIndex]) + " " + showText;
                    break;
                }
            }
        }
        return showText;
    }
    
    @Override
    public String parseShowText(final String showText) {
        String number = null;
        final int unitIndex = this.getUnitIndex();
        final int unitShowType = this.getUnitShowType();
        String unitAdd = null;
        switch (unitShowType) {
            case 0: {
                unitAdd = CurrencyCellProperty.CURRENCY_UNIT_SIGNS[unitIndex];
                if (showText.startsWith(unitAdd)) {
                    number = showText.substring(unitAdd.length());
                    break;
                }
                throw new IllegalArgumentException("开头的货币符号和定义的货币符号不符！");
            }
            case 1: {
                unitAdd = CurrencyCellProperty.CURRENCY_UNIT_NAMES[unitIndex];
                if (showText.startsWith(unitAdd)) {
                    number = showText.substring(unitAdd.length());
                    break;
                }
                throw new IllegalArgumentException("开头的货币英文缩写和定义的货币符号不符！");
            }
            case 2: {
                unitAdd = CurrencyCellProperty.CURRENCY_UNIT_NAMES[unitIndex];
                if (showText.endsWith(unitAdd)) {
                    number = showText.substring(0, showText.length() - unitAdd.length());
                    break;
                }
                throw new IllegalArgumentException("尾部的货币英文缩写和定义的货币符号不符！");
            }
            case 3: {
                unitAdd = CurrencyCellProperty.CURRENCY_UNIT_TITLES[unitIndex];
                if (showText.endsWith(unitAdd)) {
                    number = showText.substring(0, showText.length() - unitAdd.length());
                    break;
                }
                throw new IllegalArgumentException("尾部的货币名称和定义的货币符号不符！");
            }
            default: {
                number = showText;
                break;
            }
        }
        return super.parseShowText(number);
    }
}
